module Rancher
  class ConfigGenerator < Rails::Generators::Base
    source_root File.expand_path('../templates', __FILE__)
    def setup
      config = file?('rancher/rancherconfig.json') ? load_json_file('rancher/rancherconfig.json') : {}
      docker_config = file?('docker/dockerconfig.json') ? load_json_file('docker/dockerconfig.json') : {}

      @docker_compose_config = YAML.load_file(docker_config['prod_yml'])
      @services = @docker_compose_config['services'].each_with_object([]) { |(key, _value), r| r << key; }
      @url = ask_with_default('Rancher URL:[example:http://<server_ip>:8080]', config['url'] || '')
      @access_key = ask_with_default('ACCESS KEY:', config['accessKey'])
      @secret_key = ask_with_default('SECRET KEY:', config['secretKey'])
      @environment = ask_with_default('Enviroment Id Or Name:', config['environment'])
      @stack = ask_with_default('stack name:', config['stack'] || File.basename(Dir.getwd))

      @debug = false
    end

    def create_rancher_config
      template 'rancherconfig.json', 'rancher/rancherconfig.json'
    end

    def ignore_rancher_config
      ignore('.gitignore', 'rancher/rancherconfig.json')
      ignore('.dockerignore', 'lib/tasks/rancher.rake')
      ignore('.dockerignore', 'rancher/')
    end

    def create_ranche_compose
      template 'rancher-compose.yml', 'rancher/rancher-compose.yml'
    end

    def create_rancher_wrapper
      template 'rancher', 'bin/rancher'
      run('chmod +x bin/rancher')
    end

    def create_rancher_task
      template 'rancher.rake', 'lib/tasks/rancher.rake'
    end

    private

    def file?(path)
      File.exist?(path)
    end

    def ignore(filename, content)
      if file? filename
        append_to_file filename, content+"\n"
      else
        create_file filename, content
      end
    end

    def load_json_file(file)
      JSON.parse(File.read(file))
    end

    def ask_with_default(statement, default_value)
      answer = ask("#{statement}[#{default_value}]:", Thor::Shell::Color::BLUE)
      answer.present? ? answer : default_value
    end
  end
end
