module Rspec
  class ConfigGenerator < Rails::Generators::Base
    source_root File.expand_path('../templates', __FILE__)
    def setup; end

    def add_gem
      gem 'spring-commands-rspec'
      gem_group :test, :development do
        gem 'rspec-rails'
        gem 'factory_bot'
        gem 'factory_bot_rails'
        gem 'database_cleaner'
        gem 'faker'
        gem 'pry-byebug'
      end
    end

    def run_bundle
      run('bundle install')
    end

    def generate_rspec
      generate('rspec:install')
    end

    def config_spring
      # run("bundle binstubs rspec-core")
      # run("bundle exec spring binstubs rspec")
    end

    def generator_config
      factory_dir = 'spec/factories'
      empty_directory factory_dir
      content = <<-CODE
       config.generators do |g|
            g.test_framework :rspec,
                             fixtures: true,
                             view_specs: false,
                             helper_specs: false,
                             routing_specs: false,
                             request_specs: false
            g.fixture_replacement :factory_bot, dir: "#{factory_dir}"
       end
     CODE
      inject_into_file 'config/application.rb', content, after: "class Application < Rails::Application\n"
    end

    def helper_config
      strategy = if mongoid?
                   :truncation
                 else
                   :transaction
      end
      content = <<-CODE
RSpec.configure do |config|
  config.use_transactional_fixtures = false
  config.include FactoryGirl::Syntax::Methods
  config.fail_fast = true
  config.before(:suite) do
    DatabaseCleaner.strategy = :#{strategy}
  end
  config.before(:each) do
    DatabaseCleaner.start
  end
  config.after(:each) do
    DatabaseCleaner.clean_with(:truncation, except: %w[])
  end
end
CODE
      append_file 'spec/rails_helper.rb', content
    end

    private

    def file?(path)
      File.exist?(path)
    end

    def mongoid?
      file?('config/mongoid.yml')
    end
  end
end
